use rocket::{
    get,
    http::{ContentType, Status},
    State,
};

use tokio::{fs::File, io::AsyncReadExt};

use crate::library::Library;

#[get("/video/raw?<v>")]
pub async fn video_file(
    v: &str,
    library: &State<Library>,
) -> Option<(Status, (ContentType, Vec<u8>))> {
    let video = if let Some(video) = library.get_video_by_id(v).await {
        video
    } else {
        library.get_video_by_youtube_id(v).await.unwrap()
    };

    if let Ok(mut file) = File::open(&video.path).await {
        let mut buf = Vec::with_capacity(51200);
        file.read_to_end(&mut buf).await.ok()?;
        let content_type = if video.path.ends_with("mp4") {
            ContentType::new("video", "mp4")
        } else {
            ContentType::new("video", "webm")
        };

        return Some((Status::Ok, (content_type, buf)));
    }

    None
}

#[get("/video/thumbnail?<v>")]
pub async fn video_thumbnail(
    v: &str,
    library: &State<Library>,
) -> Option<(Status, (ContentType, Vec<u8>))> {
    let video = if let Some(video) = library.get_video_by_id(v).await {
        video
    } else {
        library.get_video_by_youtube_id(v).await.unwrap()
    };

    if let Some(data) = library.get_thumbnail(&video).await {
        return Some((Status::Ok, (ContentType::PNG, data)));
    }

    None
}

#[get("/favicon")]
pub async fn fav_icon() -> (Status, (ContentType, &'static [u8])) {
    (
        Status::Ok,
        (ContentType::PNG, include_bytes!("../../src/icon.png")),
    )
}
